package net.wanji.opt.service.signalcontrol.impl;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.entity.develop.servicedevelop.develop.StatusCodeEnum;
import net.wanji.feign.pojo.vo.SchemeSendVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.opt.cache.PhaseDirTurnCache;
import net.wanji.opt.dao.mapper.CrossSchemeOptLogMapper;
import net.wanji.opt.dto.CrossPhaseDTO;
import net.wanji.opt.po.base.CrossSchemeOptLogPO;
import net.wanji.opt.service.signalcontrol.FeignProxyService;
import net.wanji.opt.vo.SchemeOptSendVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;

/**
 * @author duanruiming
 * @date 2023/03/02 9:11
 */
@Service
@Slf4j
public class FeignProxyServiceImpl implements FeignProxyService {

    @Autowired
    private UtcFeignClients utcFeignClients;
    @Autowired
    private CrossSchemeOptLogMapper crossSchemeOptLogMapper;
    @Autowired
    private PhaseDirTurnCache phaseDirTurnCache;

    @Override
    public JsonViewObject schemeOptSend(SchemeOptSendVO schemeOptSendVO) throws Exception {

        SchemeSendVO schemeSendVO = new SchemeSendVO();
        BeanUtils.copyProperties(schemeOptSendVO, schemeSendVO);
        JsonViewObject jsonViewObject = utcFeignClients.schemeSend(schemeSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            log.error("实时监控路口优化方案下发远程调用异常！", jsonViewObject);
            throw new Exception();
        }

        // 将优化记录插入方案优化记录表
        List<CrossSchemeOptLogPO> crossSchemeOptLogPOS = getCrossSchemeOptLogPOList(schemeOptSendVO);
        crossSchemeOptLogMapper.insertBatch(crossSchemeOptLogPOS);
        return jsonViewObject.success("手动下发优化方案成功");
    }

    /**
     * 获取方案优化数据，插入到数据库
     *
     * @param schemeOptSendVO
     * @return
     */
    private List<CrossSchemeOptLogPO> getCrossSchemeOptLogPOList(SchemeOptSendVO schemeOptSendVO) {
        List<CrossSchemeOptLogPO> crossSchemeOptLogPOS = new ArrayList<>();
        Map<String, CrossPhaseDTO> phaseDirTurnMap = phaseDirTurnCache.getPhaseDirTurnMap();

        long dataBatchTimeLong = LocalDateTime.now().toEpochSecond(ZoneOffset.of("+8"));
        Map<String, Integer> phaseOffsetTimeMap = schemeOptSendVO.getPhaseOffsetTimeMap();
        String crossId = schemeOptSendVO.getCrossCode();
        List<SchemeSendVO.Pattern> patternList = schemeOptSendVO.getPatternList();
        for (SchemeSendVO.Pattern pattern : patternList) {
            String patternNo = pattern.getPatternNo();
            String offset = pattern.getOffset();

            List<SchemeSendVO.Pattern.Ring> rings = pattern.getRings();
            for (SchemeSendVO.Pattern.Ring ring : rings) {
                String ringNo = ring.getRingNo();

                List<SchemeSendVO.Pattern.Ring.Phase> phaseList = ring.getPhaseList();
                for (SchemeSendVO.Pattern.Ring.Phase phase : phaseList) {
                    String phaseNo = phase.getPhaseNo();
                    String greenTime = phase.getGreenTime();
                    Integer greenTimeOffset = phaseOffsetTimeMap.get(phaseNo);

                    for (Map.Entry<String, CrossPhaseDTO> entry : phaseDirTurnMap.entrySet()) {
                        CrossPhaseDTO crossPhaseDTO = entry.getValue();
                        if (StringUtils.equals(crossId, crossPhaseDTO.getCrossId()) && StringUtils.equals(phaseNo, crossPhaseDTO.getPhaseNo())) {
                            Integer dirType = crossPhaseDTO.getDirType();
                            String turnType = crossPhaseDTO.getTurnType();

                            CrossSchemeOptLogPO crossSchemeOptLogPO = new CrossSchemeOptLogPO();
                            crossSchemeOptLogPO.setCrossId(crossId);
                            crossSchemeOptLogPO.setSchemeNo(patternNo);
                            crossSchemeOptLogPO.setDirType(dirType);
                            crossSchemeOptLogPO.setTurnType(turnType);
                            crossSchemeOptLogPO.setOffset(Integer.valueOf(offset));
                            crossSchemeOptLogPO.setRingNo(Integer.valueOf(ringNo));
                            crossSchemeOptLogPO.setPhaseNo(phaseNo);
                            crossSchemeOptLogPO.setPhaseOrderId(Integer.valueOf(phase.getSort()));
                            crossSchemeOptLogPO.setOriGreenTime(Integer.valueOf(greenTime) - greenTimeOffset);
                            crossSchemeOptLogPO.setOptGreenTime(Integer.valueOf(greenTime));
                            crossSchemeOptLogPO.setOptTime(greenTimeOffset);
                            crossSchemeOptLogPO.setOptType(schemeOptSendVO.getOptType());
                            crossSchemeOptLogPO.setOptReason(schemeOptSendVO.getOptReason());
                            crossSchemeOptLogPO.setStartTime(new Date());
                            crossSchemeOptLogPO.setIsRelation(schemeOptSendVO.getRelationFlag());
                            crossSchemeOptLogPO.setRelationCrossId(schemeOptSendVO.getRelationCrossId());
                            crossSchemeOptLogPO.setOptResult("1");
                            crossSchemeOptLogPO.setOptResultDesc(StatusCodeEnum.STATUS_00200.getDetail());
                            crossSchemeOptLogPO.setDataBatchTime((int) dataBatchTimeLong);
                        }
                    }
                }
            }
        }
        return crossSchemeOptLogPOS;
    }
}
