package net.wanji.opt.service.strategy.impl;

import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import net.wanji.opt.dao.mapper.strategy.IdeaMapper;
import net.wanji.opt.dao.mapper.strategy.SceneMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyIdeaMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyMapper;
import net.wanji.opt.dao.mapper.strategy.StrategyMapper;
import net.wanji.opt.dto.IntegerIdsDTO;
import net.wanji.opt.dto.strategy.AddOrUpdateSceneDTO;
import net.wanji.opt.dto.strategy.QuerySceneDTO;
import net.wanji.opt.po.strategy.IdeaPO;
import net.wanji.opt.po.strategy.ScenePO;
import net.wanji.opt.po.strategy.SceneStrategyIdeaPO;
import net.wanji.opt.po.strategy.SceneStrategyPO;
import net.wanji.opt.po.strategy.StrategyPO;
import net.wanji.opt.service.strategy.SceneService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/2/27 11:42
 */
@Service
public class SceneServiceImpl implements SceneService {
    private final SceneMapper sceneMapper;
    private final SceneStrategyMapper sceneStrategyMapper;
    private final SceneStrategyIdeaMapper sceneStrategyIdeaMapper;
    private final StrategyMapper strategyMapper;
    private final IdeaMapper ideaMapper;

    public SceneServiceImpl(SceneMapper sceneMapper, SceneStrategyMapper sceneStrategyMapper,
                            SceneStrategyIdeaMapper sceneStrategyIdeaMapper, StrategyMapper strategyMapper, IdeaMapper ideaMapper) {
        this.sceneMapper = sceneMapper;
        this.sceneStrategyMapper = sceneStrategyMapper;
        this.sceneStrategyIdeaMapper = sceneStrategyIdeaMapper;
        this.strategyMapper = strategyMapper;
        this.ideaMapper = ideaMapper;
    }

    @Override
    @Transactional
    public void addOrUpdateScene(AddOrUpdateSceneDTO addOrUpdateSceneDTO) {
        Integer id = addOrUpdateSceneDTO.getId();
        if (ObjectUtil.isEmpty(id) || id == 0) {
            // 不传ID为新增
            // 插入场景表获取场景ID
            ScenePO scenePO = new ScenePO();
            fillData(addOrUpdateSceneDTO, scenePO);
            sceneMapper.insertOne(scenePO);
            // 插入关系表
            Integer sceneId = scenePO.getId();
            updateRelationship(addOrUpdateSceneDTO, sceneId);
        } else {
            // 传ID为修改
            // 修改场景表
            ScenePO scenePO = new ScenePO();
            scenePO.setId(id);
            fillData(addOrUpdateSceneDTO, scenePO);
            sceneMapper.updateOne(scenePO);
            // 删除关系表并重新插入
            List<Integer> sceneIdList = new ArrayList<>();
            sceneIdList.add(id);
            sceneStrategyMapper.deleteBySceneIds(sceneIdList);
            sceneStrategyIdeaMapper.deleteBySceneIds(sceneIdList);
            updateRelationship(addOrUpdateSceneDTO, id);
        }
    }

    private void updateRelationship(AddOrUpdateSceneDTO addOrUpdateSceneDTO, Integer id) {
        List<AddOrUpdateSceneDTO.StrategyListElement> strategyList = addOrUpdateSceneDTO.getStrategyList();
        for (AddOrUpdateSceneDTO.StrategyListElement strategy : strategyList) {
            SceneStrategyPO sceneStrategyPO = new SceneStrategyPO();
            sceneStrategyPO.setSceneId(id);
            Integer strategyId = strategy.getId();
            sceneStrategyPO.setStrategyId(strategyId);
            Integer priority = strategy.getPriority();
            sceneStrategyPO.setSceneStrategyPriority(priority);
            sceneStrategyMapper.insertOne(sceneStrategyPO);
            // 插入场景-策略-方法关系表
            List<AddOrUpdateSceneDTO.IdeaListElement> ideaList = strategy.getIdeaList();
            for (AddOrUpdateSceneDTO.IdeaListElement idea : ideaList) {
                SceneStrategyIdeaPO sceneStrategyIdeaPO = new SceneStrategyIdeaPO();
                sceneStrategyIdeaPO.setSceneId(id);
                sceneStrategyIdeaPO.setStrategyId(strategyId);
                Integer ideaId = idea.getId();
                sceneStrategyIdeaPO.setIdeaId(ideaId);
                sceneStrategyIdeaPO.setIdeaOrder(idea.getOrder());
                sceneStrategyIdeaMapper.insertOne(sceneStrategyIdeaPO);
            }
        }
    }

    private static void fillData(AddOrUpdateSceneDTO addOrUpdateSceneDTO, ScenePO scenePO) {
        String sceneCode = addOrUpdateSceneDTO.getSceneCode();
        scenePO.setSceneCode(sceneCode);
        String sceneName = addOrUpdateSceneDTO.getSceneName();
        scenePO.setSceneName(sceneName);
        Integer secneTarget = addOrUpdateSceneDTO.getSecneTarget();
        scenePO.setSceneTarget(secneTarget);
    }

    @Override
    @Transactional
    public void deleteScene(IntegerIdsDTO integerIdsDTO) {
        List<Integer> ids = integerIdsDTO.getIds();
        sceneMapper.deleteByIds(ids);
        sceneStrategyMapper.deleteBySceneIds(ids);
        sceneStrategyIdeaMapper.deleteBySceneIds(ids);
    }

    @Override
    public PageInfo<AddOrUpdateSceneDTO> queryScene(QuerySceneDTO querySceneDTO) {
        List<AddOrUpdateSceneDTO> res = new ArrayList<>();
        String sceneName = querySceneDTO.getSceneName();
        Integer sceneTarget = querySceneDTO.getSceneTarget();
        Integer pageNum = querySceneDTO.getPageNum();
        Integer pageSize = querySceneDTO.getPageSize();
        PageHelper.startPage(pageNum, pageSize);
        List<ScenePO> scenePOList = sceneMapper.selectByNameAndTarget(sceneName, sceneTarget);
        PageInfo<ScenePO> scenePOPageInfo = new PageInfo<>(scenePOList);
        for (ScenePO scenePO : scenePOList) {
            AddOrUpdateSceneDTO addOrUpdateSceneDTO = new AddOrUpdateSceneDTO();
            Integer id = scenePO.getId();
            addOrUpdateSceneDTO.setId(id);
            String sceneCode = scenePO.getSceneCode();
            addOrUpdateSceneDTO.setSceneCode(sceneCode);
            String sceneNameFromDb = scenePO.getSceneName();
            addOrUpdateSceneDTO.setSceneName(sceneNameFromDb);
            Integer sceneTargetFromDb = scenePO.getSceneTarget();
            addOrUpdateSceneDTO.setSecneTarget(sceneTargetFromDb);
            // 构造策略列表
            List<AddOrUpdateSceneDTO.StrategyListElement> strategyList = buildStrategyList(id);
            addOrUpdateSceneDTO.setStrategyList(strategyList);
            res.add(addOrUpdateSceneDTO);
        }
        PageInfo<AddOrUpdateSceneDTO> resPageInfo = new PageInfo<>();
        BeanUtils.copyProperties(scenePOPageInfo, resPageInfo);
        resPageInfo.setList(res);
        return resPageInfo;
    }

    private List<AddOrUpdateSceneDTO.StrategyListElement> buildStrategyList(Integer sceneId) {
        List<AddOrUpdateSceneDTO.StrategyListElement> strategyList = new ArrayList<>();
        List<SceneStrategyPO> sceneStrategyPOList = sceneStrategyMapper.selectBySceneId(sceneId);
        for (SceneStrategyPO sceneStrategyPO : sceneStrategyPOList) {
            AddOrUpdateSceneDTO.StrategyListElement strategyListElement = new AddOrUpdateSceneDTO.StrategyListElement();
            Integer strategyId = sceneStrategyPO.getStrategyId();
            strategyListElement.setId(strategyId);
            StrategyPO strategyPO = strategyMapper.selectById(strategyId);
            strategyListElement.setStrategyName(strategyPO.getStrategyName());
            strategyListElement.setPriority(sceneStrategyPO.getSceneStrategyPriority());
            // 构造方法列表
            List<AddOrUpdateSceneDTO.IdeaListElement> ideaList = buildIdeaList(sceneId, strategyId);
            strategyListElement.setIdeaList(ideaList);
            strategyList.add(strategyListElement);
        }
        List<AddOrUpdateSceneDTO.StrategyListElement> collect = strategyList.stream()
                .sorted(Comparator.comparing(AddOrUpdateSceneDTO.StrategyListElement::getPriority).reversed())
                .collect(Collectors.toList());
        return collect;
    }

    private List<AddOrUpdateSceneDTO.IdeaListElement> buildIdeaList(Integer sceneId, Integer strategyId) {
        List<AddOrUpdateSceneDTO.IdeaListElement> ideaList = new ArrayList<>();
        List<SceneStrategyIdeaPO> sceneStrategyIdeaPOList = sceneStrategyIdeaMapper
                .selectBySceneIdAndStrategyId(sceneId, strategyId);
        for (SceneStrategyIdeaPO sceneStrategyIdeaPO : sceneStrategyIdeaPOList) {
            AddOrUpdateSceneDTO.IdeaListElement ideaListElement = new AddOrUpdateSceneDTO.IdeaListElement();
            Integer ideaId = sceneStrategyIdeaPO.getIdeaId();
            IdeaPO ideaPO = ideaMapper.selectById(ideaId);
            ideaListElement.setId(ideaId);
            ideaListElement.setIdeaName(ideaPO.getIdeaName());
            Integer order = sceneStrategyIdeaPO.getIdeaOrder();
            ideaListElement.setOrder(order);
            ideaList.add(ideaListElement);
        }
        ideaList.sort(Comparator.comparing(AddOrUpdateSceneDTO.IdeaListElement::getOrder));
        return ideaList;
    }
}
