package net.wanji.opt.task;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.CrossStatusEnum;
import net.wanji.common.framework.Constants;
import net.wanji.opt.dao.mapper.trend.CrossDataRealtimeMapper;
import net.wanji.opt.dto.CrossTurnDataRealtimeDTO;
import net.wanji.opt.po.trend.CrossDataRealtimePO;
import net.wanji.opt.service.CrossOptimizeService;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author hfx
 * @date 2023/1/16 16:39
 * @desc RealtimeOptimizeTask
 */

@Component
@Slf4j
public class CrossMonitorTask {

    @Resource
    CrossOptimizeService crossOptimizeService;
    @Resource
    CrossDataRealtimeMapper crossDataRealtimeMapper;

    /**
     * 路口实时监测
     */
    @Scheduled(cron = "* 0/1 * * * ? ")
    public void realtimeMonitor() {

        log.info("路口实时监测任务...");
        // 获取路口实时监测数据
        List<CrossDataRealtimePO> crossDataRealtimePOList = listCrossDataRealtime();
        log.info("路口实时数据：{}条", crossDataRealtimePOList.size());
        // 测试时无数据暂时注释
//        if(crossDataRealtimePOList.isEmpty()) {
//            return ;
//        }

        // 获取所有信控路口异常数据（失衡/拥堵/溢出/死锁）
        List<CrossDataRealtimePO> abnormalCrossList = listAbnormalCross(crossDataRealtimePOList);
        log.info("加载异常路口：{}条", abnormalCrossList.size());
//        if(abnormalCrossList.isEmpty()) {
//            return ;
//        }

        // 调用路口实时优化
        crossOptimizeService.realtimeOptimize(abnormalCrossList, crossDataRealtimePOList);

    }

    /**
     * 获取异常路口数据
     *
     * @return
     */
    public List<CrossDataRealtimePO> listAbnormalCross(List<CrossDataRealtimePO> crossDataRealtimePOList) {

        List<CrossDataRealtimePO> dtoList = new ArrayList<>();
        CrossDataRealtimePO dto = null;
        int status = 0;
        for (CrossDataRealtimePO po : crossDataRealtimePOList) {

            // 获取失衡、拥堵、溢出路口数据
            if (Constants.SystemParam.NO.equals(po.getIsSpillover()) && Constants.SystemParam.NO.equals(po.getIsCongestion()) && Constants.SystemParam.NO.equals(po.getIsUnbalance())) {
                continue;
            }

            if (Constants.SystemParam.YES.equals(po.getIsSpillover())) {         // 溢出
                status = CrossStatusEnum.SPILLOVER.getCode();
            } else if (Constants.SystemParam.YES.equals(po.getIsCongestion())) { // 拥堵
                status = CrossStatusEnum.CONGESTION.getCode();
            } else if (Constants.SystemParam.YES.equals(po.getIsUnbalance())) {  // 失衡
                status = CrossStatusEnum.UNBALANCE.getCode();
            }
            dto = new CrossDataRealtimePO();
            BeanUtils.copyProperties(po, dto);
            dto.setCrossId(po.getCrossId());
            dto.setStatus(status);
            dto.setDirList(null);
            dto.setTurnList(null);
            dtoList.add(dto);
        }
        return dtoList;
    }

    /**
     * 获取路口实时监测数据
     * （暂定从数据中心kafka获取）
     *
     * @return
     */
    public List<CrossDataRealtimePO> listCrossDataRealtime() {

        // todo test数据
        List<CrossDataRealtimePO> crossDataRealtimePOList = new ArrayList<>(1);
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectById("14Q1409IRF0");
        CrossDataRealtimePO crossDataRealtimePO1 = crossDataRealtimeMapper.selectById("14Q0S09IQB0");
        crossDataRealtimePOList.add(crossDataRealtimePO);
        crossDataRealtimePOList.add(crossDataRealtimePO1);


        // todo  对数据进行处理，可能主要是CrossTurnDataRealtimeDTO中的车间距，车时距等配置修改
        CrossTurnDataRealtimeDTO crossTurnDataRealtimeDTO = new CrossTurnDataRealtimeDTO();
        // todo 通过获取场景，策略，方法参数，判断下发方案数据
        return crossDataRealtimePOList;
    }

}